package Wx

import (
	"context"
	"math/rand"
	"strconv"
	"time"
	"wechat-api/ent"
	"wechat-api/ent/predicate"
	"wechat-api/ent/wx"
	"wechat-api/hook"
	"wechat-api/internal/svc"
	"wechat-api/internal/types"
	"wechat-api/internal/utils/dberrorhandler"

	"github.com/suyuan32/simple-admin-common/msg/errormsg"

	"github.com/suyuan32/simple-admin-common/utils/pointy"
	"github.com/zeromicro/go-zero/core/logx"
)

type GetWxListLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGetWxListLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetWxListLogic {
	return &GetWxListLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *GetWxListLogic) GetWxList(req *types.WxListReq) (*types.WxListResp, error) {
	organizationId := l.ctx.Value("organizationId").(uint64)

	servers, err := l.svcCtx.DB.Server.Query().All(l.ctx)
	serverSet := make(map[uint64]*ent.Server, len(servers))
	for _, s := range servers {
		serverSet[s.ID] = s
	}

	var predicates []predicate.Wx
	predicates = append(predicates, wx.OrganizationIDEQ(organizationId))
	if req.Port != nil {
		predicates = append(predicates, wx.PortContains(*req.Port))
	}
	if req.ProcessId != nil {
		predicates = append(predicates, wx.ProcessIDContains(*req.ProcessId))
	}
	if req.Callback != nil {
		predicates = append(predicates, wx.CallbackContains(*req.Callback))
	}
	data, err := l.svcCtx.DB.Wx.Query().Where(predicates...).WithAgent().Page(l.ctx, req.Page, req.PageSize)

	if err != nil {
		return nil, dberrorhandler.DefaultEntError(l.Logger, err, req)
	}

	resp := &types.WxListResp{}
	resp.Msg = errormsg.Success
	resp.Data.Total = data.PageDetails.Total

	for _, v := range data.List {
		// 创建 hookClient 客户端
		serverInfo := serverSet[v.ServerID]
		hookClient := hook.NewHook(serverInfo.PrivateIP, serverInfo.AdminPort, v.Port)

		// 获取登录状态
		loginInfo, err := hookClient.IsLoginStatus()
		var loginStatus uint8 = 0
		if err != nil {
			l.Error("退出登录失败", err)
		} else {
			if loginInfo.Onlinestatus == "3" {
				loginStatus = 1
			}
		}

		processID := v.ProcessID
		wxid := v.Wxid
		account := v.Account
		nickname := v.Nickname
		tel := v.Tel
		headBig := v.HeadBig

		if loginStatus == 1 {
			// 如果处于登录状态，获取登录信息
			wxInfo, _ := hookClient.GetSelfLoginInfo()
			if err != nil {
				l.Error("退出登录失败", err)
			} else {
				processID = wxInfo.ProcessID
				wxid = wxInfo.Wxid
				account = wxInfo.Account
				nickname = wxInfo.Nickname
				tel = wxInfo.Tel
				headBig = wxInfo.HeadBig
				_ = l.svcCtx.DB.Wx.UpdateOneID(v.ID).
					SetNotNilStatus(&loginStatus).
					SetNotNilProcessID(&wxInfo.ProcessID).
					SetNotNilWxid(&wxInfo.Wxid).
					SetNotNilAccount(&wxInfo.Account).
					SetNotNilNickname(&wxInfo.Nickname).
					SetNotNilTel(&wxInfo.Tel).
					SetNotNilHeadBig(&wxInfo.HeadBig).
					Exec(l.ctx)
			}
		} else {
			if loginStatus != v.Status {
				wxid = strconv.Itoa(rand.New(rand.NewSource(time.Now().UnixNano())).Intn(1000000))
				_ = l.svcCtx.DB.Wx.UpdateOneID(v.ID).
					SetNotNilStatus(&loginStatus).
					SetWxid(wxid).
					Exec(l.ctx)
			}
		}

		var agent types.AgentInfo
		if v.Edges.Agent != nil {
			agent = types.AgentInfo{
				BaseIDInfo: types.BaseIDInfo{
					Id:        &v.AgentID,
					CreatedAt: pointy.GetPointer(v.Edges.Agent.CreatedAt.UnixMilli()),
					UpdatedAt: pointy.GetPointer(v.Edges.Agent.UpdatedAt.UnixMilli()),
				},
				Name:       &v.Edges.Agent.Name,
				Role:       &v.Edges.Agent.Role,
				Status:     &v.Edges.Agent.Status,
				Background: &v.Edges.Agent.Background,
				Examples:   &v.Edges.Agent.Examples,
			}
		}

		resp.Data.Data = append(resp.Data.Data,
			types.WxInfo{
				BaseIDInfo: types.BaseIDInfo{
					Id:        &v.ID,
					CreatedAt: pointy.GetPointer(v.CreatedAt.UnixMilli()),
					UpdatedAt: pointy.GetPointer(v.UpdatedAt.UnixMilli()),
				},
				Status:         &loginStatus,
				ServerId:       &v.ServerID,
				Port:           &v.Port,
				ProcessId:      &processID,
				Callback:       &v.Callback,
				Wxid:           &wxid,
				Account:        &account,
				Nickname:       &nickname,
				Tel:            &tel,
				HeadBig:        &headBig,
				OrganizationId: &v.OrganizationID,
				AgentId:        &v.AgentID,
				AgentInfo:      &agent,
			})
	}

	return resp, nil
}
